// if you want to skip the messages on the serial monitor comment the macro below
//#define  _DEBUG_MESSAGE

#define  BUTTON_PRESSED  (!(PINE & 0x04))
#define  LED_OFF     0
#define  LED_ON      1
#define  LED_TOGGLE  2

bool LED_Status, Button_Status;
int Temperature, Potentiometer;

void Connect_to_WiFi_Network (char Net[], char Pass[])
{
  String Command;
  Command = "AT+CWJAP=\"";
  Command = Command + Net;
  Command = Command + "\",\"";
  Command = Command + Pass;
  Command = Command + "\"\r";
  Serial1.print (Command);
}

void Disconnect_from_WiFi_Network (void)
{
  Serial1.print ("AT+CWQAP\r");
}

// using PIC-WEB as a server
// the default server IP address is 192.168.0.130
// if you want to change it edit the macro below
#define  IP_Server  "192.168.0.130"
void Update_Server_Status (bool &LED, bool &Button, int &Temperature, int &Potentometer)
{
  String Cmd, IP;
  long int Timeout = 100000;
  int Var=0;
  char c;
  IP = "AT+CIPSTART=\"TCP\",\"";
  IP += IP_Server;
  IP += "\",80";
  Serial1.println (IP);
  #ifdef  _DEBUG_MESSAGE
  Serial.println (IP);  // debug purpose
  #endif
  if (Serial1.find ("Error"))
    return;
  Cmd = "GET /status.xml HTTP/1.0\r\n\r\n";  //GET / HTTP/1.0
  Serial1.print("AT+CIPSEND=");
  Serial1.println (Cmd.length());
  if (Serial1.find (">"))
  {
    Serial1.print(Cmd);
    #ifdef  _DEBUG_MESSAGE
    Serial.print ('>');
    Serial.print(Cmd);
    #endif
  }
  else
  {
    Serial1.println ("AT+CIPCLOSE");
    #ifdef  _DEBUG_MESSAGE
    Serial.println ("Connection Timeout!");
    #endif
    delay (1000);
    return;
  }
  
  
  // original but delay is too long and during that time the data
  // sent by the server to the ESP module are lost;
  // replaced by timeout instead with value 100K
  // updated after each character
  //delay (2000);

  while (1)
  {
    switch (Var)
    {
      case 0:
        Serial1.find ("<led1>");
        delay (1);
        if (Serial1.available())
          LED = !(Serial1.read () & 0x0F);
        Var++;
      break;
      case 1:
        Serial1.find ("<btn0>");
        delay (1);
        if (Serial1.available())
          if (Serial1.read () == 'd')
            Button = 1;
          else
            Button = 0;
        Var++;
      break;
      case 2:
        Serial1.find ("<temp0>");
        delay (1);
        Temperature = 0;
        while (Serial1.available ())
        {
          c = Serial1.read ();
          if (c == '<')
            break;
          Temperature = Temperature * 10 + (c & 0x0F);
        }
        Var++;
      break;
      case 3:
        Serial1.find ("<pot0>");
        delay (1);
        Potentometer = 0;
        while (Serial1.available ())
        {
          c = Serial1.read ();
          if (c == '<')
            break;
          Potentometer = Potentometer * 10 + (c & 0x0F);
        }
        Var++;
      break;
      default:
        while (Timeout--)
        {
          if (Serial1.available ())
          {
            char c = Serial1.read ();
            #ifdef  _DEBUG_MESSAGE
            Serial.print ((char) c);
            #endif
            Timeout = 50000;
          }
        }
        #ifdef  _DEBUG_MESSAGE
        Serial.println ("Transfer complete!");
        #endif
        return;
      break;
    }
  }
}

void setup()
{
  // LEDs on Olimexino set as outputs
  pinMode (7, OUTPUT);
  digitalWrite (7, LOW);
  pinMode (9, OUTPUT);
  digitalWrite (9, HIGH);

  DDRE = DDRE & (~(1<<2));  // Button - input 

  pinMode (8, OUTPUT);    // UEXT Power
  digitalWrite (8, LOW);  // UEXT Power turned on
  
  #ifdef  _DEBUG_MESSAGE
  Serial.begin (115200);
  #endif
  Serial1.begin (115200);

  delay (1000);
  Serial1.print ("AT+CWMODE=3\r");
  delay (1000);
  Serial1.print ("AT+RST\r");

  Disconnect_from_WiFi_Network ();
  Connect_to_WiFi_Network ("OlimexTenda", "DarmadaniaSarsapat");
}

bool Button = true;  // this variable is used to avoid command repetition due to holding the button
int Val=0;

void loop ()
{
  if (BUTTON_PRESSED && Button)
  {
    // LEDs indicates whether geting data is active
    // green LED - ON; yellow LED - OFF
    Val = !Val;
    digitalWrite (7, Val);
    digitalWrite (9, !Val);

    Button = false;  // mark button as pressed
  }

  if (!BUTTON_PRESSED)
    Button = true;  // mark button as released

  if (Val)
  {
    // sending command to get data for the peripherals
    Update_Server_Status (LED_Status, Button_Status, Temperature, Potentiometer);
    Serial.print ("LED status: "); Serial.println (LED_Status);
    Serial.print ("Button status: "); Serial.println (Button_Status);
    Serial.print ("Temperature: "); Serial.println (Temperature);
    Serial.print ("Potentiometer: "); Serial.println (Potentiometer);
    Serial.println ();
  }
}

